import celery
from spylunking.log.setup_logging import build_colorized_logger


class CustomTask(celery.Task):
    """CustomTask"""

    log_label = "custom_task"

    def on_success(self, retval, task_id, args, kwargs):
        """on_success

        http://docs.celeryproject.org/en/latest/reference/celery.app.task.html

        :param retval: return value
        :param task_id: celery task id
        :param args: arguments passed into task
        :param kwargs: keyword arguments passed into task
        """
        log = build_colorized_logger(
            name='task.on_success')

        log.info(("{} SUCCESS - retval={} task_id={} "
                  "args={} kwargs={}")
                 .format(
                     self.log_label,
                     retval,
                     task_id,
                     args,
                     kwargs))
    # end of on_success

    def on_failure(self, exc, task_id, args, kwargs, einfo):
        """on_failure

        http://docs.celeryproject.org/en/latest/userguide/tasks.html#task-inheritance

        :param exc: exception
        :param task_id: task id
        :param args: arguments passed into task
        :param kwargs: keyword arguments passed into task
        :param einfo: exception info
        """
        log = build_colorized_logger(
            name='task.on_failure')
        use_exc = str(exc)
        log.error(("{} FAIL - exc={} "
                   "args={} kwargs={}")
                  .format(
                     self.log_label,
                     use_exc,
                     args,
                     kwargs))
    # end of on_failure

# end of CustomTask



import os
import celery
from spylunking.log.setup_logging import build_colorized_logger


def get_celery_app(
        name=os.getenv(
            "CELERY_NAME",
            "worker"),
        auth_url=os.getenv(
            "BROKER_URL",
            "redis://localhost:6379/9"),
        backend_url=os.getenv(
            "BACKEND_URL",
            "redis://localhost:6379/10"),
        include_tasks=[],
        ssl_options=None,
        transport_options=None,
        path_to_config_module=os.getenv(
            "CONFIG_MODULE_PATH",
            "celery_loaders.work_tasks.celery_config"),
        worker_log_format=os.getenv(
            "WORKER_LOG_FORMAT",
            "%(asctime)s: %(levelname)s %(message)s"),
        **kwargs):
    """get_celery_app

    :param name: name for this app
    :param auth_url: celery broker
    :param backend_url: celery backend
    :param include_tasks: list of modules containing tasks to add
    :param ssl_options: security options dictionary
    :param trasport_options: transport options dictionary
    :param path_to_config_module: config module
    :param worker_log_format: format for logs
    """
    log = build_colorized_logger(
        name='get_celery_app')

    if len(include_tasks) == 0:
        log.error(("creating celery app={} MISSING tasks={}")
                  .format(
                    name,
                    include_tasks))
    else:
        log.info(("creating celery app={} tasks={}")
                 .format(
                    name,
                    include_tasks))

    # get the Celery application
    app = celery.Celery(
        name,
        broker_url=auth_url,
        result_backend=backend_url,
        include=include_tasks)

    app.config_from_object(
        path_to_config_module,
        namespace="CELERY")

    app.conf.update(kwargs)

    if transport_options:
        log.info(("loading transport_options={}")
                 .format(transport_options))
        app.conf.update(**transport_options)
    # custom tranport options

    if ssl_options:
        log.info(("loading ssl_options={}")
                 .format(ssl_options))
        app.conf.update(**ssl_options)
    # custom ssl options

    if len(include_tasks) > 0:
        app.autodiscover_tasks(include_tasks)

    return app
# end of get_celery_app



#!/usr/bin/env python

import os
from spylunking.log.setup_logging import build_colorized_logger
from celery import signals
from celery_loaders.work_tasks.get_celery_app import get_celery_app


# Disable celery log hijacking
# https://github.com/celery/celery/issues/2509
@signals.setup_logging.connect
def setup_celery_logging(**kwargs):
    pass


name = "worker"
log = build_colorized_logger(
    name=name)

log.info(("start - {}")
         .format(name))

broker_url = os.getenv(
    "WORKER_BROKER_URL",
    "redis://localhost:6379/9").strip().lstrip()
backend_url = os.getenv(
    "WORKER_BACKEND_URL",
    "redis://localhost:6379/10").strip().lstrip()

# comma delimited list of task module files:
tasks_str = os.getenv(
    "WORKER_TASKS",
    ("celery_loaders.work_tasks.tasks,"
     "celery_loaders.work_tasks.always_fails_tasks"))
include_tasks = tasks_str.split(",")

ssl_options = {}
transport_options = {}

log.info(("broker={} backend={} include_tasks={}")
         .format(
            broker_url,
            backend_url,
            include_tasks))

# Get the Celery app from the project's get_celery_app module
app = get_celery_app(
    name=name,
    auth_url=broker_url,
    backend_url=backend_url,
    include_tasks=include_tasks,
    worker_hijack_root_logger=False)

log.info("starting celery")
app.start()

log.info(("end - {}")
         .format(name))



from celery.task import task
from spylunking.log.setup_logging import build_colorized_logger
from celery_loaders.work_tasks.custom_task import CustomTask


@task(
    bind=True,
    base=CustomTask,
    queue="always_fails")
def always_fails(
        self,
        work_dict):
    """always_fails

    :param work_dict: dictionary for key/values
    """

    log = build_colorized_logger(
        name='always_fails_tasks')

    label = "always_fails"

    log.info(("task - {} - start "
              "work_dict={}")
             .format(label,
                     work_dict))

    raise Exception(
            work_dict.get(
                "test_failure",
                "simulating a failure"))

    log.info(("task - {} - done")
             .format(label))

    return True
# end of always_fails



import uuid
from celery.task import task
from spylunking.log.setup_logging import build_colorized_logger
from celery_loaders.work_tasks.custom_task import CustomTask


@task(
    bind=True,
    base=CustomTask,
    queue="do_some_work")
def do_some_work(
        self,
        work_dict):
    """do_some_work

    :param work_dict: dictionary for key/values
    """

    log = build_colorized_logger(
        name='do_some_work')

    label = "do_some_work"

    log.info(("task - {} - start "
              "work_dict={}")
             .format(label,
                     work_dict))

    ret_data = {
        "job_results": ("some response key={}").format(
                            str(uuid.uuid4()))
    }

    log.info(("task - {} - result={} done")
             .format(
                 ret_data,
                 label))

    return ret_data
# end of do_some_work



import os
import sys
import warnings
import unittest

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

try:
    from distutils.command.build_py import build_py_2to3 as build_py
except ImportError:
    from distutils.command.build_py import build_py


cur_path, cur_script = os.path.split(sys.argv[0])
os.chdir(os.path.abspath(cur_path))

install_requires = [
    "celery>=4.1.0",
    "colorlog",
    "coverage",
    "flake8>=3.4.1",
    "future",
    "kombu>=4.1.0",
    "mock",
    "pep8>=1.7.1",
    "pylint",
    "redis",
    "spylunking",
    "unittest2"
]


if sys.version_info < (2, 7):
    warnings.warn(
        "Python 2.6 is not supported.",
        DeprecationWarning)


def celery_loaders_test_suite():
    test_loader = unittest.TestLoader()
    test_suite = test_loader.discover("tests", pattern="test_*.py")
    return test_suite


# Don"t import celery_loaders module here, since deps may not be installed
sys.path.insert(0, os.path.join(os.path.dirname(__file__), "celery_loaders"))

setup(
    name="celery-loaders",
    cmdclass={"build_py": build_py},
    version="1.0.6",
    description="Celery Application and Task Loader Examples",
    long_description="Examples for loading celery applications with " +
    "easy-to-discover task modules",
    author="Jay Johnson",
    author_email="jay.p.h.johnson@gmail.com",
    url="https://github.com/jay-johnson/celery-loaders",
    packages=[
        "celery_loaders",
        "celery_loaders.log",
        "celery_loaders.work_tasks",
    ],
    package_data={},
    install_requires=install_requires,
    test_suite="setup.celery_loaders_test_suite",
    tests_require=[
    ],
    scripts=[
        "./run-celery-task.py"
    ],
    use_2to3=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ])



broker_url = "redis://localhost:6379/9"
result_backend = "redis://localhost:6379/10"

# http://docs.celeryproject.org/en/latest/userguide/optimizing.html

# these are targeted at optimizing processing
# on long-running tasks
# while increasing reliability

# http://docs.celeryproject.org/en/latest/userguide/configuration.html#std:setting-worker_prefetch_multiplier  # noqa
worker_hijack_root_logger = False

# http://docs.celeryproject.org/en/latest/userguide/configuration.html#std:setting-worker_prefetch_multiplier  # noqa
worker_prefetch_multiplier = 1

# http://docs.celeryproject.org/en/latest/userguide/configuration.html#std:setting-broker_heartbeat  # noqa
broker_heartbeat = 240  # seconds

# http://docs.celeryproject.org/en/latest/userguide/configuration.html#std:setting-broker_connection_max_retries  # noqa
broker_connection_max_retries = None

# http://docs.celeryproject.org/en/latest/userguide/configuration.html#std:setting-task_acks_late  # noqa
task_acks_late = True

# http://docs.celeryproject.org/en/latest/userguide/calling.html#calling-retry
task_publish_retry_policy = {
    "interval_max": 1,
    "max_retries": 120,     # None = forever
    "interval_start": 0.1,
    "interval_step": 0.2}

task_serializer = "json"
result_serializer = "json"
accept_content = ["json"]
timezone = "America/Los_Angeles"

task_routes = {
    "run.check_values": "low-priority",
    "run.calculate_results": "high-priority"}



#!/usr/bin/env python

import os
import sys
import json
import argparse
from celery import signals
from spylunking.log.setup_logging import build_colorized_logger
from celery_loaders.work_tasks.get_celery_app import get_celery_app


name = 'run-celery-task'
log = build_colorized_logger(name=name)

parser = argparse.ArgumentParser(description="sending Celery task data")
parser.add_argument(
    "-f",
    help="task data file: path to data file",
    required=True,
    dest="data_file")
parser.add_argument(
    "-t",
    help="task name: celery_loaders.work_tasks.tasks.do_some_work",
    required=True,
    dest="task_name")
parser.add_argument(
    "-w",
    "--time-for-result",
    help="get the task's result after waiting (default 1.0 seconds)",
    required=False,
    dest="time_for_result")
args = parser.parse_args()


task_name = args.task_name
task_data = None
get_task_result = False
time_to_wait_for_result = 1.0
if args.data_file:
    if os.path.exists(args.data_file):
        task_data = json.loads(open(args.data_file).read())
# end of loading the data to send

if args.time_for_result:
    get_task_result = True
    try:
        time_to_wait_for_result = float(args.time_for_result)
    except Exception as e:
        time_to_wait_for_result = 1.0
        log.error(("Invalid time_for_result={} "
                   "please use a float or int")
                  .format(
                      args.time_for_result))
    # end of try/ex
# end of if waiting for the result

if not task_data:
    log.error(("Please provide a "
               "path to task_data file with -f <path to file>"))
    sys.exit(1)
# end of checking if there is data to send


# Disable celery log hijacking
# https://github.com/celery/celery/issues/2509
@signals.setup_logging.connect
def setup_celery_logging(**kwargs):
    pass


log.info(("start - {}")
         .format(name))

broker_url = os.getenv(
    "BROKER_URL",
    "redis://localhost:6379/9").strip().lstrip()
backend_url = os.getenv(
    "BACKEND_URL",
    "redis://localhost:6379/10").strip().lstrip()

# comma delimited
tasks_str = os.getenv(
    "INCLUDE_TASKS",
    "celery_loaders.work_tasks.tasks")
include_tasks = tasks_str.split(",")

log.info(("connecting Celery={} "
          "broker={} backend={} tasks={}")
         .format(
            name,
            broker_url,
            backend_url,
            include_tasks))

# Get the Celery app using the celery-loaders api
app = get_celery_app(
        name,
        auth_url=broker_url,
        backend_url=backend_url,
        include_tasks=include_tasks)

log.info(("app.broker_url={} calling task={} data={}")
         .format(
            app.conf.broker_url,
            task_name,
            task_data))
task_job = app.send_task(
    task_name,
    (task_data,))
log.info(("calling task={} - started job_id={}")
         .format(
            task_name,
            task_job.id))

if get_task_result:
    log.info(("task={} - waiting seconds={} "
              "for results")
             .format(
                 task_job.id,
                 time_to_wait_for_result))
    task_job.get(timeout=time_to_wait_for_result)
    log.info(("task={} - success "
              "job_id={} task_result={}")
             .format(
                task_name,
                task_job.id,
                task_job.result))
# end of if getting task result is enabled

log.info(("end - {}")
         .format(name))

sys.exit(0)
